@@ -22,25 +22,30 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#include "config.h"
+
 #include <unistd.h>
 #include <errno.h>
 #include <stdio.h>
 #include <signal.h>
 #include <string.h>
 #include <ctype.h>
-#ifndef __FreeBSD__
+#if !defined(__FreeBSD__) && !defined(__MidnightBSD__)
 #include <alloca.h>
 #endif
 #include <getopt.h>
 #include <fnmatch.h>
+#ifdef HAVE_READLINE
 #include <readline/readline.h>
 #include <readline/history.h>
+#endif
+#include <locale.h>
 
 #if !defined(FNM_EXTMATCH)
 #define FNM_EXTMATCH 0
 #endif
 
-#define spa_debug(fmt,...) printf(fmt, ## __VA_ARGS__)
+#define spa_debug(fmt,...) printf(fmt"\n", ## __VA_ARGS__)
 
 #include <spa/utils/result.h>
 #include <spa/utils/string.h>
@@ -132,6 +137,11 @@ struct command {
 	bool (*func) (struct data *data, const char *cmd, char *args, char **error);
 };
 
+static struct spa_dict * global_props(struct global *global);
+static struct global * obj_global(struct remote_data *rd, uint32_t id);
+static int children_of(struct remote_data *rd, uint32_t parent_id,
+	const char *child_type, uint32_t **children);
+
 static int pw_split_ip(char *str, const char *delimiter, int max_tokens, char *tokens[])
 {
 	const char *state = NULL;
@@ -301,7 +311,12 @@ static void on_core_info(void *_data, const struct pw_core_info *info)
 static void set_prompt(struct remote_data *rd)
 {
 	snprintf(prompt, sizeof(prompt), "%s>> ", rd->name);
+#ifdef HAVE_READLINE
 	rl_set_prompt(prompt);
+#else
+	printf("%s", prompt);
+	fflush(stdout);
+#endif
 }
 
 static void on_core_done(void *_data, uint32_t id, int seq)
@@ -800,9 +815,9 @@ static void info_endpoint_stream(struct proxy_data *pd)
 	info->change_mask = 0;
 }
 
-static void core_event_info(void *object, const struct pw_core_info *info)
+static void core_event_info(void *data, const struct pw_core_info *info)
 {
-	struct proxy_data *pd = object;
+	struct proxy_data *pd = data;
 	struct remote_data *rd = pd->rd;
 	if (pd->info)
 		printf("remote %d core %d changed\n", rd->id, info->id);
@@ -821,9 +836,9 @@ static const struct pw_core_events core_events = {
 };
 
 
-static void module_event_info(void *object, const struct pw_module_info *info)
+static void module_event_info(void *data, const struct pw_module_info *info)
 {
-	struct proxy_data *pd = object;
+	struct proxy_data *pd = data;
 	struct remote_data *rd = pd->rd;
 	if (pd->info)
 		printf("remote %d module %d changed\n", rd->id, info->id);
@@ -841,9 +856,9 @@ static const struct pw_module_events module_events = {
 	.info = module_event_info
 };
 
-static void node_event_info(void *object, const struct pw_node_info *info)
+static void node_event_info(void *data, const struct pw_node_info *info)
 {
-	struct proxy_data *pd = object;
+	struct proxy_data *pd = data;
 	struct remote_data *rd = pd->rd;
 	if (pd->info)
 		printf("remote %d node %d changed\n", rd->id, info->id);
@@ -856,10 +871,10 @@ static void node_event_info(void *object, const struct pw_node_info *info)
 	}
 }
 
-static void event_param(void *object, int seq, uint32_t id,
+static void event_param(void *_data, int seq, uint32_t id,
 		uint32_t index, uint32_t next, const struct spa_pod *param)
 {
-        struct proxy_data *data = object;
+        struct proxy_data *data = _data;
 	struct remote_data *rd = data->rd;
 
 	if (rd->data->interactive)
@@ -876,9 +891,9 @@ static const struct pw_node_events node_events = {
 };
 
 
-static void port_event_info(void *object, const struct pw_port_info *info)
+static void port_event_info(void *data, const struct pw_port_info *info)
 {
-	struct proxy_data *pd = object;
+	struct proxy_data *pd = data;
 	struct remote_data *rd = pd->rd;
 	if (pd->info)
 		printf("remote %d port %d changed\n", rd->id, info->id);
@@ -897,9 +912,9 @@ static const struct pw_port_events port_events = {
 	.param = event_param
 };
 
-static void factory_event_info(void *object, const struct pw_factory_info *info)
+static void factory_event_info(void *data, const struct pw_factory_info *info)
 {
-	struct proxy_data *pd = object;
+	struct proxy_data *pd = data;
 	struct remote_data *rd = pd->rd;
 	if (pd->info)
 		printf("remote %d factory %d changed\n", rd->id, info->id);
@@ -917,9 +932,9 @@ static const struct pw_factory_events factory_events = {
 	.info = factory_event_info
 };
 
-static void client_event_info(void *object, const struct pw_client_info *info)
+static void client_event_info(void *data, const struct pw_client_info *info)
 {
-	struct proxy_data *pd = object;
+	struct proxy_data *pd = data;
 	struct remote_data *rd = pd->rd;
 	if (pd->info)
 		printf("remote %d client %d changed\n", rd->id, info->id);
@@ -932,10 +947,10 @@ static void client_event_info(void *object, const struct pw_client_info *info)
 	}
 }
 
-static void client_event_permissions(void *object, uint32_t index,
+static void client_event_permissions(void *_data, uint32_t index,
 		uint32_t n_permissions, const struct pw_permission *permissions)
 {
-        struct proxy_data *data = object;
+        struct proxy_data *data = _data;
 	struct remote_data *rd = data->rd;
 	uint32_t i;
 
@@ -958,9 +973,9 @@ static const struct pw_client_events client_events = {
 	.permissions = client_event_permissions
 };
 
-static void link_event_info(void *object, const struct pw_link_info *info)
+static void link_event_info(void *data, const struct pw_link_info *info)
 {
-	struct proxy_data *pd = object;
+	struct proxy_data *pd = data;
 	struct remote_data *rd = pd->rd;
 	if (pd->info)
 		printf("remote %d link %d changed\n", rd->id, info->id);
@@ -979,9 +994,9 @@ static const struct pw_link_events link_events = {
 };
 
 
-static void device_event_info(void *object, const struct pw_device_info *info)
+static void device_event_info(void *data, const struct pw_device_info *info)
 {
-	struct proxy_data *pd = object;
+	struct proxy_data *pd = data;
 	struct remote_data *rd = pd->rd;
 	if (pd->info)
 		printf("remote %d device %d changed\n", rd->id, info->id);
@@ -1007,10 +1022,10 @@ static void session_info_free(struct pw_session_info *info)
 	free(info);
 }
 
-static void session_event_info(void *object,
+static void session_event_info(void *data,
 				const struct pw_session_info *update)
 {
-	struct proxy_data *pd = object;
+	struct proxy_data *pd = data;
 	struct remote_data *rd = pd->rd;
 	struct pw_session_info *info = pd->info;
 
@@ -1054,10 +1069,10 @@ static void endpoint_info_free(struct pw_endpoint_info *info)
 	free(info);
 }
 
-static void endpoint_event_info(void *object,
+static void endpoint_event_info(void *data,
 				const struct pw_endpoint_info *update)
 {
-	struct proxy_data *pd = object;
+	struct proxy_data *pd = data;
 	struct remote_data *rd = pd->rd;
 	struct pw_endpoint_info *info = pd->info;
 
@@ -1108,10 +1123,10 @@ static void endpoint_stream_info_free(struct pw_endpoint_stream_info *info)
 	free(info);
 }
 
-static void endpoint_stream_event_info(void *object,
+static void endpoint_stream_event_info(void *data,
 				const struct pw_endpoint_stream_info *update)
 {
-	struct proxy_data *pd = object;
+	struct proxy_data *pd = data;
 	struct remote_data *rd = pd->rd;
 	struct pw_endpoint_stream_info *info = pd->info;
 
@@ -1497,15 +1512,68 @@ static bool do_destroy(struct data *data, const char *cmd, char *args, char **er
 	return true;
 }
 
+static struct global *
+obj_global_port(struct remote_data *rd, struct global *global, const char *port_direction, const char *port_id)
+{
+	struct global *global_port_found = NULL;
+	uint32_t *ports = NULL;
+	int port_count;
+
+	port_count = children_of(rd, global->id, PW_TYPE_INTERFACE_Port, &ports);
+
+	if (port_count <= 0)
+		return NULL;
+
+	for (int i = 0; i < port_count; i++) {
+		struct global *global_port = obj_global(rd, ports[i]);
+
+		if (!global_port)
+			continue;
+
+		struct spa_dict *props_port = global_props(global_port);
+
+		if (spa_streq(spa_dict_lookup(props_port, "port.direction"), port_direction)
+				&& spa_streq(spa_dict_lookup(props_port, "port.id"), port_id)) {
+			global_port_found = global_port;
+			break;
+		}
+	}
+
+	free(ports);
+	return global_port_found;
+}
+
+static void create_link_with_properties(struct data *data, struct pw_properties *props)
+{
+	struct remote_data *rd = data->current;
+	uint32_t id;
+	struct pw_proxy *proxy;
+	struct proxy_data *pd;
+
+	proxy = (struct pw_proxy*)pw_core_create_object(rd->core,
+					  "link-factory",
+					  PW_TYPE_INTERFACE_Link,
+					  PW_VERSION_LINK,
+					  props ? &props->dict : NULL,
+					  sizeof(struct proxy_data));
+
+	pd = pw_proxy_get_user_data(proxy);
+	pd->rd = rd;
+	pd->proxy = proxy;
+	pd->class = &link_class;
+	pw_proxy_add_object_listener(proxy, &pd->object_listener, &link_events, pd);
+	pw_proxy_add_listener(proxy, &pd->proxy_listener, &proxy_events, pd);
+
+	id = pw_map_insert_new(&data->vars, proxy);
+	printf("%d = @proxy:%d\n", id, pw_proxy_get_id((struct pw_proxy*)proxy));
+}
+
 static bool do_create_link(struct data *data, const char *cmd, char *args, char **error)
 {
 	struct remote_data *rd = data->current;
 	char *a[5];
-        int n;
-	uint32_t id;
-	struct pw_proxy *proxy;
+	int n;
 	struct pw_properties *props = NULL;
-	struct proxy_data *pd;
 
 	n = pw_split_ip(args, WHITESPACE, 5, a);
 	if (n < 4) {
@@ -1526,25 +1594,55 @@ static bool do_create_link(struct data *data, const char *cmd, char *args, char
 	if (!spa_streq(a[3], "-"))
 		pw_properties_set(props, PW_KEY_LINK_INPUT_PORT, a[3]);
 
-	proxy = (struct pw_proxy*)pw_core_create_object(rd->core,
-					  "link-factory",
-					  PW_TYPE_INTERFACE_Link,
-					  PW_VERSION_LINK,
-					  props ? &props->dict : NULL,
-					  sizeof(struct proxy_data));
+	if (spa_streq(a[1], "*") && spa_streq(a[3], "*")) {
+		struct global *global_out, *global_in;
+		struct proxy_data *pd_out, *pd_in;
+		uint32_t n_output_ports, n_input_ports;
+
+		global_out = find_global(rd, a[0]);
+		if (global_out == NULL) {
+			*error = spa_aprintf("%s: unknown global '%s'", cmd, a[0]);
+			return false;
+		}
+		global_in = find_global(rd, a[2]);
+		if (global_in == NULL) {
+			*error = spa_aprintf("%s: unknown global '%s'", cmd, a[2]);
+			return false;
+		}
+
+		pd_out = pw_proxy_get_user_data(global_out->proxy);
+		pd_in = pw_proxy_get_user_data(global_in->proxy);
+
+		n_output_ports = ((struct pw_node_info *)pd_out->info)->n_output_ports;
+		n_input_ports = ((struct pw_node_info *)pd_in->info)->n_input_ports;
+
+		if (n_output_ports != n_input_ports) {
+			*error = spa_aprintf("%s: Number of ports don't match (%u != %u)", cmd, n_output_ports, n_input_ports);
+			return false;
+		}
+
+		for (uint32_t i = 0; i < n_output_ports; i++) {
+			char port_id[4];
+			struct global *global_port_out, *global_port_in;
+
+			snprintf(port_id, 4, "%d", i);
+
+			global_port_out = obj_global_port(rd, global_out, "out", port_id);
+			global_port_in = obj_global_port(rd, global_in, "in", port_id);
+
+			if (!global_port_out || !global_port_in)
+				continue;
+
+			pw_properties_setf(props, PW_KEY_LINK_OUTPUT_PORT, "%d", global_port_out->id);
+			pw_properties_setf(props, PW_KEY_LINK_INPUT_PORT, "%d", global_port_in->id);
+
+			create_link_with_properties(data, props);
+		}
+	} else
+		create_link_with_properties(data, props);
 
 	pw_properties_free(props);
 
-	pd = pw_proxy_get_user_data(proxy);
-	pd->rd = rd;
-	pd->proxy = proxy;
-        pd->class = &link_class;
-        pw_proxy_add_object_listener(proxy, &pd->object_listener, &link_events, pd);
-        pw_proxy_add_listener(proxy, &pd->proxy_listener, &proxy_events, pd);
-
-	id = pw_map_insert_new(&data->vars, proxy);
-	printf("%d = @proxy:%d\n", id, pw_proxy_get_id((struct pw_proxy*)proxy));
-
 	return true;
 }
 
@@ -2951,18 +3049,20 @@ static bool parse(struct data *data, char *buf, char **error)
 }
 
 /* We need a global variable, readline doesn't have a closure arg */
-static struct data *readline_dataptr;
+static struct data *input_dataptr;
 
-static void readline_process_line(char *line)
+static void input_process_line(char *line)
 {
-	struct data *d = readline_dataptr;
+	struct data *d = input_dataptr;
 	char *error;
 
 	if (!line)
 		line = strdup("quit");
 
 	if (line[0] != '\0') {
+#ifdef HAVE_READLINE
 		add_history(line);
+#endif
 		if (!parse(d, line, &error)) {
 			fprintf(stderr, "Error: \"%s\"\n", error);
 			free(error);
@@ -2976,8 +3076,21 @@ static void do_input(void *data, int fd, uint32_t mask)
 	struct data *d = data;
 
 	if (mask & SPA_IO_IN) {
-		readline_dataptr = d;
+		input_dataptr = d;
+#ifdef HAVE_READLINE
 		rl_callback_read_char();
+#else
+		{
+			char *line = NULL;
+			size_t s = 0;
+
+			if (getline(&line, &s, stdin) < 0) {
+				free(line);
+				line = NULL;
+			}
+			input_process_line(line);
+		}
+#endif
 
 		if (d->current == NULL)
 			pw_main_loop_quit(d->loop);
@@ -2989,6 +3102,7 @@ static void do_input(void *data, int fd, uint32_t mask)
 	}
 }
 
+#ifdef HAVE_READLINE
 static char *
 readline_match_command(const char *text, int state)
 {
@@ -3030,13 +3144,14 @@ readline_command_completion(const char *text, int start, int end)
 static void readline_init()
 {
 	rl_attempted_completion_function = readline_command_completion;
-	rl_callback_handler_install(">> ", readline_process_line);
+	rl_callback_handler_install(">> ", input_process_line);
 }
 
 static void readline_cleanup()
 {
 	rl_callback_handler_remove();
 }
+#endif
 
 static void do_quit_on_signal(void *data, int signal_number)
 {
@@ -3063,22 +3178,24 @@ int main(int argc, char *argv[])
 	struct pw_loop *l;
 	char *opt_remote = NULL;
 	char *error;
-	bool daemon = false;
+	bool daemon = false, monitor = false;
 	struct remote_data *rd;
 	static const struct option long_options[] = {
-		{ "help",	no_argument,		 NULL, 'h' },
-		{ "version",	no_argument,		 NULL, 'V' },
-		{ "daemon",	no_argument,		 NULL, 'd' },
-		{ "remote",	required_argument,	 NULL, 'r' },
+		{ "help",	no_argument,		NULL, 'h' },
+		{ "version",	no_argument,		NULL, 'V' },
+		{ "monitor",	no_argument,		NULL, 'm' },
+		{ "daemon",	no_argument,		NULL, 'd' },
+		{ "remote",	required_argument,	NULL, 'r' },
 		{ NULL,	0, NULL, 0}
 	};
 	int c, i;
 
 	setlinebuf(stdout);
 
+	setlocale(LC_ALL, "");
 	pw_init(&argc, &argv);
 
-	while ((c = getopt_long(argc, argv, "hVdr:", long_options, NULL)) != -1) {
+	while ((c = getopt_long(argc, argv, "hVmdr:", long_options, NULL)) != -1) {
 		switch (c) {
 		case 'h':
 			show_help(&data, argv[0], false);
@@ -3094,6 +3211,9 @@ int main(int argc, char *argv[])
 		case 'd':
 			daemon = true;
 			break;
+		case 'm':
+			monitor = true;
+			break;
 		case 'r':
 			opt_remote = optarg;
 			break;
@@ -3138,13 +3258,17 @@ int main(int argc, char *argv[])
 		printf("Welcome to PipeWire version %s. Type 'help' for usage.\n",
 				pw_get_library_version());
 
+#ifdef HAVE_READLINE
 		readline_init();
+#endif
 
 		pw_loop_add_io(l, STDIN_FILENO, SPA_IO_IN|SPA_IO_HUP, false, do_input, &data);
 
 		pw_main_loop_run(data.loop);
 
+#ifdef HAVE_READLINE
 		readline_cleanup();
+#endif
 	} else {
 		char buf[4096], *p, *error;
 
@@ -3160,9 +3284,11 @@ int main(int argc, char *argv[])
 			fprintf(stderr, "Error: \"%s\"\n", error);
 			free(error);
 		}
-		if (!data.quit && data.current) {
+		while (!data.quit && data.current) {
 			data.current->prompt_pending = pw_core_sync(data.current->core, 0, 0);
 			pw_main_loop_run(data.loop);
+			if (!monitor)
+				break;
 		}
 	}
 	spa_list_consume(rd, &data.remotes, link)
